import { Filter } from './filter';
import { MergeOperator } from './merge-operator';
import { TaskSearchRequestBody } from './task-search-request-body';
import { CaseSearchRequestBody } from './case-search-request-body';
import { FilterType } from './filter-type';
import { Params } from '../../resources/resource-provider.service';
import { MergedFilterParams } from './merged-filter-params';
/**
 * Represents an union or an intersection of {@link SimpleFilter}s and allows limited manipulation with it.
 */
export declare class MergedFilter extends Filter {
    protected _operator: MergeOperator;
    /**
     * @ignore
     * Stores the underlying filter data
     */
    private readonly _filters;
    /**
     * Creates a filter with the provided body and the provided type
     * @param filterBody the body of the new filter
     * @param operator the merge operator that is used to combine the filters
     * @param type the type of the new filter. Type must match the type of the body.
     * @returns a filter with the given body, operator and type. `id` of the new filter is set to an empty string.
     */
    static fromQuery(filterBody: Array<CaseSearchRequestBody> | Array<TaskSearchRequestBody>, operator: MergeOperator, type: FilterType): Filter;
    /**
     * Creates a case filter with the provided body
     * @param filterBody the body of the new filter
     * @param operator the merge operator that is used to combine the filters
     * @returns a case filter with the given body and operator. `id` of the new filter is set to an empty string.
     */
    static fromCaseQuery(filterBody: Array<CaseSearchRequestBody>, operator: MergeOperator): Filter;
    /**
     * Creates a task filter with the provided body
     * @param filterBody the body of the new filter
     * @param operator the merge operator that is used to combine the filters
     * @returns a task filter with the given body and operator. `id` of the new filter is set to an empty string.
     */
    static fromTaskQuery(filterBody: Array<TaskSearchRequestBody>, operator: MergeOperator): Filter;
    /**
     * @param id identifier of the filter
     * @param type type of resources that the filter can query
     * @param filterBodies `Array` of search requests matching this filter's type
     * @param _operator boolean operator that is used to combine the filters
     * @param title human readable filter name
     */
    constructor(id: string, type: FilterType, filterBodies: Array<CaseSearchRequestBody> | Array<TaskSearchRequestBody>, _operator: MergeOperator, title?: string);
    set operator(operator: MergeOperator);
    get operator(): MergeOperator;
    /**
     * See [Filter.clone()]{@link Filter#clone}
     */
    clone(): Filter;
    /**
     * Filters that are merged must share the same type and the same merge operator.
     *
     * See [Filter.merge()]{@link Filter#merge} for information about parameters.
     */
    merge(filter: Filter, operator: MergeOperator): MergedFilter;
    /**
     * See [Filter.getRequestBody()]{@link Filter#getRequestBody}
     */
    getRequestBody(): Array<TaskSearchRequestBody> | Array<CaseSearchRequestBody>;
    /**
     * See [Filter.bodyContainsQuery()]{@link Filter#bodyContainsQuery}
     */
    bodyContainsQuery(): boolean;
    /**
     * Returns the necessary request params for the filter.
     * @returns params with `operation` set to either `AND` or `OR` based on this object's `_operator` property.
     *
     * See {@link MergedFilterParams} for more information.
     */
    getRequestParams(): MergedFilterParams & Params;
}
