import { Filter } from './filter';
import { MergeOperator } from './merge-operator';
import { MergedFilter } from './merged-filter';
import { TaskSearchRequestBody } from './task-search-request-body';
import { CaseSearchRequestBody } from './case-search-request-body';
import { FilterType } from './filter-type';
/**
 * Holds backend search request data and allows limited manipulation with it.
 */
export declare class SimpleFilter extends Filter {
    /**
     * @ignore
     * Stores the underlying filter data
     */
    private readonly _filter;
    /**
     * Create an empty filter of provided type
     * @param type type of resources that the filter can query
     * @returns a filter with empty body of the provided type. `id` of the new filter is set to an empty string.
     */
    static empty(type: FilterType): Filter;
    /**
     * Equivalent to calling `SimpleFilter.empty(FilterType.CASE)`.
     * @returns a filter with empty body that can be used to search cases
     */
    static emptyCaseFilter(): Filter;
    /**
     * Equivalent to calling `SimpleFilter.empty(FilterType.TASK)`.
     * @returns a filter with empty body that can be used to search tasks
     */
    static emptyTaskFilter(): Filter;
    /**
     * Creates a filter with the provided body and the provided type
     * @param filterBody the body of the new filter
     * @param type the type of the new filter. Type must match the type of the body.
     * @returns a filter with the given body and type. `id` of the new filter is set to an empty string.
     */
    static fromQuery(filterBody: CaseSearchRequestBody | TaskSearchRequestBody, type: FilterType): Filter;
    /**
     * Creates a case filter with the provided body
     * @param filterBody the body of the new filter
     * @returns a case filter with the given body. `id` of the new filter is set to an empty string.
     */
    static fromCaseQuery(filterBody: CaseSearchRequestBody): Filter;
    /**
     * Creates a task filter with the provided body
     * @param filterBody the body of the new filter
     * @returns a task filter with the given body. `id` of the new filter is set to an empty string.
     */
    static fromTaskQuery(filterBody: TaskSearchRequestBody): Filter;
    /**
     * @param id identifier of the filter
     * @param type type of resources that the filter can query
     * @param filterBody search request matching this filter's `type`
     * @param title human readable filter name
     */
    constructor(id: string, type: FilterType, filterBody: CaseSearchRequestBody | TaskSearchRequestBody, title?: string);
    /**
     * See [Filter.clone()]{@link Filter#clone}
     */
    clone(): Filter;
    /**
     * See [Filter.merge()]{@link Filter#merge}
     */
    merge(filter: Filter, operator: MergeOperator): MergedFilter;
    /**
     * See [Filter.bodyContainsQuery()]{@link Filter#bodyContainsQuery}
     */
    bodyContainsQuery(): boolean;
    /**
     * See [Filter.getRequestBody()]{@link Filter#getRequestBody}
     */
    getRequestBody(): TaskSearchRequestBody | CaseSearchRequestBody;
}
