import { OnDestroy, Type } from '@angular/core';
import { Observable, ReplaySubject, Subject } from 'rxjs';
import { CaseResourceService } from '../resources/engine-endpoint/case-resource.service';
import { TaskResourceService } from '../resources/engine-endpoint/task-resource.service';
import { SearchService } from '../search/search-service/search.service';
import { ProcessService } from '../process/process.service';
import { LoggerService } from '../logger/services/logger.service';
import { Task } from '../resources/interface/task';
import { CallChainService } from '../utility/call-chain/call-chain.service';
import { TaskSetDataRequestBody, TaskSetDataRequestFields } from '../resources/interface/task-set-data-request-body';
import { Category } from '../search/models/category/category';
import { Net } from '../process/net';
import { SideMenuService } from '../side-menu/services/side-menu.service';
import { ComponentType } from '@angular/cdk/portal';
import { FilterType } from './models/filter-type';
import { Filter } from './models/filter';
import { SavedFilterMetadata } from '../search/models/persistance/saved-filter-metadata';
import { FilterMetadata } from '../search/models/persistance/filter-metadata';
import { CategoryResolverService } from '../search/category-factory/category-resolver.service';
import { DataGroup } from '../resources/interface/data-groups';
import { MatDialog } from '@angular/material/dialog';
import * as i0 from "@angular/core";
/**
 * Service that manages filters created by users of the application.
 */
export declare class UserFiltersService implements OnDestroy {
    protected _caseService: CaseResourceService;
    protected _taskService: TaskResourceService;
    protected _processService: ProcessService;
    protected _callChainService: CallChainService;
    protected _sideMenuService: SideMenuService;
    protected _log: LoggerService;
    protected _categoryResolverService: CategoryResolverService;
    protected _dialog: MatDialog;
    protected _saveFilterComponent: ComponentType<unknown>;
    protected _loadFilterComponent: ComponentType<unknown>;
    protected _initialized$: ReplaySubject<boolean>;
    protected _filterNet: Net;
    constructor(_caseService: CaseResourceService, _taskService: TaskResourceService, _processService: ProcessService, _callChainService: CallChainService, _sideMenuService: SideMenuService, _log: LoggerService, _categoryResolverService: CategoryResolverService, _dialog: MatDialog, _saveFilterComponent: ComponentType<unknown>, _loadFilterComponent: ComponentType<unknown>);
    ngOnDestroy(): void;
    /**
     * Deletes the specified filter case by its case id.
     * @param filterCaseId StringId of the filter case that should be deleted
     * @returns an Observable that emits `true` if the case was successfully deleted and `false` otherwise
     */
    delete(filterCaseId: string): Observable<boolean>;
    /**
     * Opens a side menu with filter cases and allows the user to select one of them.
     *
     * The default filter constrains the cases to be instances of the filter process and to be filters of the specified type
     * (`Case` or `Task`).
     *
     * @param filterType whether `Case` or `Task` user filters should be loaded
     * @param additionalFilter additional constrains on the displayed user filter cases. Must be of type `Case`
     * otherwise the method throws an error. If it is a {@link MergedFilter} must use the `AND` {@link MergeOperator} otherwise an error
     * will be thrown
     * @returns an Observable that emits the data necessary to reconstruct the selected filter, or `undefined` if no filter was selected
     */
    load(filterType: FilterType, additionalFilter?: Filter): Observable<any>;
    /**
     * Saves the predicate filter contained in the provided {@link SearchService} instance.
     *
     * The base filter of the search service is not saved.
     *
     * A new filter case is created and the necessary filter information is set into it.
     *
     * Then a side menu with the filter is opened for the user to enter the name and other properties.
     * The user can confirm or cancel the save by finishing the task in the side menu, or by canceling/closing.
     *
     * @param searchService search service containing a predicate filter, we want to save
     * @param allowedNets allowed nets of the view, that contains the search service
     * @param searchCategories search categories available in the saved advanced search component
     * @param viewId the viewId of the view which contained the filter.
     * If neither the `viewId` nor the `navigationItemTaskData` attribute is set the filter will be created as a new root filter
     * without any parent filter link. This will most likely result in the filter being less restrictive than what
     * the user sees in the current view, since the base filter of the view will NOT be saved alongside the saved filter.
     * @param additionalData set data request body, that is sent to the filter in addition to the default body.
     * The default body is applied first and can be overridden by this argument.
     * @param withDefaultCategories Whether the saved filter should be saved with
     * the [defaultSearchCategories]{@link FilterMetadata#defaultSearchCategories} flag set to `true`, or `false`.
     * @param inheritAllowedNets Whether the saved filter should merge its allowed nets with the allowed nets provided by
     * the {@link BaseAllowedNetsService}.
     * @param navigationItemTaskData if provided then it implies the filter has a parent filter that created its host dynamic view.
     * If this is the case the case ID of the parent filter will be set into the newly created filter case.
     * If not set it implies this is a filter originating in an in-app view and its view id will be set into the created filter.
     * @returns an observable that emits the id of the created Filter case instance or `undefined` if the user canceled the save process,
     * or the filter could not be saved
     */
    save(searchService: SearchService, allowedNets: ReadonlyArray<string>, searchCategories: ReadonlyArray<Type<Category<any>>>, viewId?: string, additionalData?: TaskSetDataRequestFields, withDefaultCategories?: boolean, inheritAllowedNets?: boolean, navigationItemTaskData?: Array<DataGroup>): Observable<SavedFilterMetadata>;
    /**
     * Saves the predicate filter contained in the provided {@link SearchService} instance.
     *
     * The base filter of the search service is not saved.
     *
     * A new filter case is created and the necessary filter information is set into it.
     *
     * @param searchService search service containing a predicate filter, we want to save
     * @param allowedNets allowed nets of the view, that contains the search service
     * @param searchCategories search categories available in the saved advanced search component
     * @param viewId the viewId of the view which contained the filter.
     * If neither the `viewId` nor the `navigationItemTaskData` attribute is set the filter will be created as a new root filter
     * without any parent filter link. This will most likely result in the filter being less restrictive than what
     * the user sees in the current view, since the base filter of the view will NOT be saved alongside the saved filter.
     * @param additionalData set data request body, that is sent to the filter in addition to the default body.
     * The default body is applied first and can be overridden by this argument.
     * @param withDefaultCategories Whether the saved filter should be saved with
     * the [defaultSearchCategories]{@link FilterMetadata#defaultSearchCategories} flag set to `true`, or `false`.
     * @param inheritAllowedNets Whether the saved filter should merge its allowed nets with the allowed nets provided by
     * the {@link BaseAllowedNetsService}.
     * @param navigationItemTaskData if provided then it implies the filter has a parent filter that created its host dynamic view.
     * If this is the case the case ID of the parent filter will be set into the newly created filter case.
     * If not set it implies this is a filter originating in an in-app view and its view id will be set into the created filter.
     * @returns an observable that emits the id of the created Filter case instance
     */
    createFilterCaseAndSetData(searchService: SearchService, allowedNets: ReadonlyArray<string>, searchCategories: ReadonlyArray<Type<Category<any>>>, viewId?: string, additionalData?: TaskSetDataRequestFields, withDefaultCategories?: boolean, inheritAllowedNets?: boolean, navigationItemTaskData?: Array<DataGroup>): Observable<string>;
    protected whenInitialized(onSuccess: () => void, method?: string, onFailure?: () => void): void;
    protected assignSetDataFinish(task: Task, data: TaskSetDataRequestBody, callChain: Subject<boolean>): void;
    protected filterMetadataFromSearchService(searchService: SearchService, searchCategories: ReadonlyArray<Type<Category<any>>>, withDefaultCategories: boolean, inheritAllowedNets: boolean): FilterMetadata;
    static ɵfac: i0.ɵɵFactoryDeclaration<UserFiltersService, [null, null, null, null, null, null, null, null, { optional: true; }, { optional: true; }]>;
    static ɵprov: i0.ɵɵInjectableDeclaration<UserFiltersService>;
}
