import { Category } from './category';
import { Operator } from '../operator/operator';
import { LoggerService } from '../../../logger/services/logger.service';
import { SearchAutocompleteOption } from './search-autocomplete-option';
import { BehaviorSubject, Observable } from 'rxjs';
import { AutocompleteOptions } from './autocomplete-options';
import { FormControl } from '@angular/forms';
import { OperatorService } from '../../operator-service/operator.service';
/**
 * Represents a Search Category whose values are a known set. The value selection is done with an autocomplete field.
 *
 * @typeparam T type of the object that the autocomplete option values use and in turn is used to generate queries
 */
export declare abstract class AutocompleteCategory<T> extends Category<Array<T>> implements AutocompleteOptions {
    /**
     * Autocomplete categories usually require a map to represent mapping of display names
     * of their options to objects that are needed to construct their queries.
     *
     * You are not required to use this map when you implement your own Category,
     * but it comes with helpful functions and might save you some time.
     *
     * See [options()]{@link AutocompleteCategory#options} and [addToMap()]{@link AutocompleteCategory#addToMap}.
     */
    protected _optionsMap: Map<string, Array<T>>;
    protected _options$: BehaviorSubject<Array<SearchAutocompleteOption<Array<T>>>>;
    private readonly _timeoutId;
    protected constructor(elasticKeywords: Array<string>, allowedOperators: Array<Operator<any>>, translationPath: string, log: LoggerService, operatorService: OperatorService);
    destroy(): void;
    /**
     * Options the user can select from for this search Category.
     */
    get options(): Array<SearchAutocompleteOption<Array<T>>>;
    /**
     * An Observable of the options the user can select from for this search Category.
     *
     * The Observable is updated when the base set of options changes, not when the available options are
     * filtered, see [filterOptions()]{@link AutocompleteCategory#filterOptions}.
     *
     * The Observable initially contains an empty array and the options may appear at a later point in time
     * (based on the concrete AutocompleteCategory subclass implementation)
     */
    get options$(): Observable<Array<SearchAutocompleteOption<Array<T>>>>;
    /**
     * Publishes a new value in the [_options$]{@link AutocompleteCategory#_options$} Subject generated from the
     * [_optionsMap]{@link AutocompleteCategory#_optionsMap}.
     *
     * The default implementation iterates trough the [_optionsMap]{@link AutocompleteCategory#_optionsMap} and
     * creates options with the keys as [text]{@link SearchAutocompleteOption#text}
     * and values as [value]{@link SearchAutocompleteOption#value}.
     */
    protected updateOptions(): void;
    /**
     * Populates the [_options]{@link AutocompleteCategory#_options} field with options for this category.
     *
     * You must call this method yourself in an appropriate place of Category creation.
     */
    protected abstract createOptions(): void;
    /**
     * The function that is used to filter shown options in the autocomplete field.
     *
     * By default the options that start with the input string are returned (case insensitive).
     *
     * By default a new value is emitted, whenever either the user input or the base set of options changes.
     * @param userInput user search input
     * @returns options that satisfy the autocomplete condition
     */
    filterOptions(userInput: Observable<string | SearchAutocompleteOption<Array<T>>>): Observable<Array<SearchAutocompleteOption<Array<T>>>>;
    /**
     * Adds a new entry or pushes value into an existing entry.
     * When a new entry is created, it is created as an Array of one element.
     * @param key where in the map should the value be added
     * @param value the value that should be added to the map
     */
    protected addToMap(key: any, value: T): void;
    protected isOperandValueSelected(newValue: SearchAutocompleteOption<Array<T>> | string): boolean;
    /**
     * Performs a transformation of the `FormControl` value before passing it into the selected `Operator` for query generation.
     * It is mostly useful only for AutocompleteCategories, where the selected value of the FormControl is an object.
     *
     * The default AutocompleteCategory implementation returns the {@link SearchAutocompleteOption} `value` attribute.
     * @param value the FormControlValue
     * @returns the value used for query generation
     */
    protected transformCategoryValue(value: SearchAutocompleteOption<Array<T>>): Array<T>;
    protected serializeOperandValue(valueFormControl: FormControl): unknown;
    /**
     * @param text the serialized output of the [serializeOperandValue()]{@link Category#serializeOperandValue} method
     * @returns the deserialized value, that can be set as FormControl value
     *
     * This method throws na error if the serialized value is not one of the autocomplete options.
     */
    protected deserializeOperandValue(text: unknown): Observable<any>;
    protected createSearchAutocompleteOption(text: string, value: Array<T>): SearchAutocompleteOption<Array<T>>;
}
