import { EventEmitter, OnDestroy } from '@angular/core';
import { DatafieldGridLayoutElement } from '../model/datafield-grid-layout-element';
import { FieldConverterService } from '../services/field-converter.service';
import { TaskContentService } from '../services/task-content.service';
import { PaperViewService } from '../../navigation/quick-panel/components/paper-view.service';
import { LoggerService } from '../../logger/services/logger.service';
import { TaskEventNotification } from '../model/task-event-notification';
import { TaskEventService } from '../services/task-event.service';
import { DataGroup } from '../../resources/interface/data-groups';
import { DataField } from '../../data-fields/models/abstract-data-field';
import { DataGroupCompact, DataGroupHideEmptyRows, DataGroupLayout, DataGroupLayoutType } from '../../resources/interface/data-group-layout';
import { FieldAlignment } from '../../resources/interface/field-alignment';
import { FieldTypeResource } from '../model/field-type-resource';
import { LoadingEmitter } from '../../utility/loading-emitter';
import { BehaviorSubject, Observable, Subscription } from 'rxjs';
import { AsyncRenderingConfiguration } from '../model/async-rendering-configuration';
import { TaskRefField } from '../../data-fields/task-ref-field/model/task-ref-field';
import { SplitDataGroup } from '../model/split-data-group';
import { Subgrid } from '../model/subgrid';
import { IncrementingCounter } from '../../utility/incrementing-counter';
import { PreprocessedDataGroups } from '../model/preprocessed-data-groups';
import { Component as DataRefComponent } from '../../data-fields/models/component';
import * as i0 from "@angular/core";
export declare abstract class AbstractTaskContentComponent implements OnDestroy {
    protected _fieldConverter: FieldConverterService;
    taskContentService: TaskContentService;
    protected _paperView: PaperViewService;
    protected _logger: LoggerService;
    protected _taskEventService: TaskEventService;
    readonly DEFAULT_LAYOUT_TYPE = DataGroupLayoutType.LEGACY;
    readonly DEFAULT_FIELD_ALIGNMENT = FieldAlignment.CENTER;
    readonly DEFAULT_ASYNC_RENDERING_CONFIGURATION: AsyncRenderingConfiguration;
    readonly DEFAULT_COMPACT_DIRECTION = DataGroupCompact.NONE;
    readonly DEFAULT_HIDE_EMPTY_ROWS = DataGroupHideEmptyRows.ALL;
    /**
     * Indicates whether data is being loaded from backend, or if it is being processed.
     */
    loading$: LoadingEmitter;
    /**
     * Emits `true` if there is at least one data field, that should be displayed. Emits `false` otherwise.
     */
    hasDataToDisplay$: Observable<boolean>;
    /**
     * Exists to allow references to the enum in the HTML
     */
    fieldTypeResource: typeof FieldTypeResource;
    /**
     * The translate text that should be displayed when the task contains no data.
     *
     * If a falsy value is provided the default text is displayed
     */
    noDataText: string;
    /**
     * The icon that should be displayed when the task contains no data.
     *
     * If a falsy value is provided the default icon is displayed
     */
    noDataIcon: string;
    /**
     * Whether an icon should be displayed when the no data message is shown.
     *
     * An icon is displayed by default
     */
    displayNoDataIcon: boolean;
    /**
     * Emits notifications about task events
     */
    taskEvent: EventEmitter<TaskEventNotification>;
    /**
     * The data fields that are currently displayed
     */
    protected _dataSource$: BehaviorSubject<Array<Subgrid>>;
    protected _subTaskContent: Subscription;
    protected _subTaskEvent: Subscription;
    protected _asyncRenderingConfig: AsyncRenderingConfiguration;
    protected _asyncRenderTimeout: number;
    protected _defaultAlignment: FieldAlignment;
    protected _defaultCompactDirection: DataGroupCompact;
    protected _defaultHideEmptyRows: DataGroupHideEmptyRows;
    protected _defaultNumberOfCols: number;
    protected _subgridIdCounter: IncrementingCounter;
    protected _existingSubgridIds: Set<string>;
    /**
     * Defines the row height of one row in task content
     */
    protected rowHeight: number;
    protected constructor(_fieldConverter: FieldConverterService, taskContentService: TaskContentService, _paperView: PaperViewService, _logger: LoggerService, _taskEventService?: TaskEventService, asyncRenderingConfiguration?: AsyncRenderingConfiguration);
    ngOnDestroy(): void;
    get taskId(): string;
    get dataSource(): Array<Subgrid>;
    /**
     * @returns the number of columns as specified by the tasks layout property, or 4 if no value is specified.
     */
    getNumberOfFormColumns(): number;
    isPaperView(): boolean;
    /**
     * @param item an object containing grid element information
     * @returns the Angular flex layouts alignment property for individual data fields based on
     * the layout configuration specified by the process
     */
    getItemAlignment(item: DatafieldGridLayoutElement): string;
    /**
     * Computes component height if the rows attribute of task layout is set.
     */
    resolveContentHeightCss(): string;
    /**
     * Decides whether there is a fixed number of rows and it need-s scrolling
     */
    scrollingNeeded(): boolean;
    /**
     * @returns the default alignment for the displayed task that can be overridden by fields individual property.
     * If te task specifies no default value the [global default]{@link AbstractTaskContentComponent#DEFAULT_FIELD_ALIGNMENT}
     * value is returned.
     */
    protected computeDefaultLayoutConfiguration(): void;
    /**
     * Computes the layout data for the tasks grid layout and sets the result to the public properties
     * of this class that are then bound to HTML.
     * @param dataGroups the data groups that should be laid out
     */
    computeLayoutData(dataGroups: Array<DataGroup>): void;
    protected renderFields(subgrids: Map<string, Subgrid>): void;
    protected spreadFieldRenderingOverTime(subgrids: Array<Subgrid>, iteration?: number): void;
    /**
     * Clones the content of the data groups to prevent unintentional memory accesses to source data.
     * Rearranges the data groups to accommodate taskrefs. Filters out hidden and forbidden fields.
     * Populates dashboard task ref fields with dashboard tile data.
     * @param dataGroups
     * @returns the preprocessed data groups
     */
    protected preprocessDataGroups(dataGroups: Array<DataGroup>): Array<DataGroup>;
    /**
     * Clones the content of the data groups to prevent unintentional memory accesses to source data.
     * Rearranges the data groups to accommodate taskrefs.
     * Determines if the data groups contain a dashboard task ref field.
     * @param dataGroups
     * @returns the preprocessed data groups with metadata
     */
    protected rearrangeDataGroups(dataGroups: Array<DataGroup>): PreprocessedDataGroups;
    /**
     * Creates a duplicate of the provided data group array.
     *
     * Only the data groups are cloned, the fields are only copied as references.
     * @param dataGroups the data groups that should be cloned
     * @returns the duplicated data groups
     */
    protected cloneDataGroups(dataGroups: Array<DataGroup>): Array<DataGroup>;
    /**
     * Passes over all the fields in the provided data groups and if they are visible, initializes their local layout attribute.
     * @param dataGroups the containers of the fields that should have their local layout initialized
     */
    protected initializeLocalFieldLayout(dataGroups: Array<DataGroup>): void;
    /**
     * Creates a duplicate of the provided data group array and filters away any fields and data groups that are marked as hidden.
     * Because of the duplication the filtering doesn't affect the original instances and they remain unchanged.
     * @param dataGroups the data groups that should be filtered
     * @returns the filtered duplicated data groups
     */
    protected cloneAndFilterHidden(dataGroups: Array<DataGroup>): Array<DataGroup>;
    protected isTaskRef(field: DataField<unknown>): boolean;
    /**
     * Sorts the input data group based on the Y coordinate of the fields and splits it into parts on the first task ref.
     * If some fields appear before the first task ref they are extracted into a new [startGroup]{@link SplitDataGroup#startGroup}.
     * If some fields appear after the first task ref they are extracted into a new [endGroup]{@link SplitDataGroup#endGroup}.
     * @param dataGroup
     * @protected
     */
    protected splitDataGroupOnTaskRef(dataGroup: DataGroup): SplitDataGroup;
    /**
     * Creates a new data group that contains only the passed task ref.
     * The information about nesting and parent task/case are preserved from the passed original data group.
     * @param originalDataGroup source of the information about parent task/case
     * @param taskRef the task ref that will be added to the new data group
     * @param title optional title for taskref
     * @protected
     */
    protected createComponentTaskRefDataGroup(originalDataGroup: DataGroup, taskRef: TaskRefField, title?: string): DataGroup;
    /**
     * Identifies data groups that represent dashboard tiles,
     * removes them from the data group array and passes them onto the task ref field instance.
     * @param preprocessedDataGroups
     * @param rawDataGroups
     * @returns an array of data group objects that does not contain dashboard tiles referenced by the dashboard task ref
     * @protected
     */
    protected preprocessDashboardTaskRef(preprocessedDataGroups: Array<DataGroup>, rawDataGroups: Array<DataGroup>): Array<DataGroup>;
    /**
     * Computes the layout data for a single data group with grid layout. The resulting layout is saved into the input objects.
     * @param dataGroup the data group that should be laid out into a grid
     * @param subgrid the object that holds the context of the computed layouting unit
     */
    protected computeGridLayout(dataGroup: DataGroup, subgrid: Subgrid): void;
    /**
     * Applies the empty row removal and field compacting rules based on the provided layout configuration.
     *
     * The input grid is modified in place.
     * @param grid the state of the grid that should be modified
     * @param layout configuration of the applied compacting rules
     * @param subgrid the object that holds the context of the computed layouting unit
     */
    protected collapseGridEmptySpace(grid: Array<Array<string>>, layout: DataGroupLayout, subgrid: Subgrid): void;
    /**
     * Moves any element as far UP as it can go. Elements that were originally declared in the same row might end up in different rows.
     * Resulting trailing empty rows are removed.
     *
     * The input grid is modified in place.
     * @param grid the state of the grid that should be modified
     * @param hideRows configuration for empty row removal during the compacting process
     * @param subgrid the object that holds the context of the computed layouting unit
     */
    protected compactFieldsUp(grid: Array<Array<string>>, hideRows: DataGroupHideEmptyRows, subgrid: Subgrid): void;
    /**
     * Removes rows from the grid that only contain empty elements (are empty). The grid is modified in place.
     * @param grid the grid that should have it's empty rows removed
     * @param firstRow the 0 based index of the first row that should be checked. Use 0 to start from the beginning of the grid.
     *
     * If no value is provided, the grid will be checked from the first row.
     *
     * @param lastRow the 0 based index of the row where the checking should end. The row with this index is not checked.
     *
     * If no value is provided the entire grid from the `firstRow` will be checked.
     *
     * If a value that is smaller or equal to the `firstRow` is provided no checks will be preformed.
     */
    protected removeEmptyRows(grid: Array<Array<string>>, firstRow?: number, lastRow?: number): void;
    /**
     * Determines the dimensions of the element in the grid with its top-left corner at the specified position
     * @param grid the grid of elements
     * @param x the X coordinate of the desired elements top-left corner
     * @param y the Y coordinate of the desired elements top-left corner
     * @param subgrid the object that holds the context of the computed layouting unit
     * @returns the width and height of the specified element
     */
    protected getElementDimensions(grid: Array<Array<string>>, x: number, y: number, subgrid: Subgrid): {
        width: number;
        height: number;
    };
    /**
     * Determines whether the specified area in the grid contains no elements, or not
     * @param grid the grid of elements
     * @param x the X coordinate of the tested areas top-left corner
     * @param y the Y coordinate of the tested areas top-left corner
     * @param width the width of the tested area
     * @param height the height of the tested area
     */
    protected isAreaEmpty(grid: Array<Array<string>>, x: number, y: number, width: number, height: number): boolean;
    /**
     * Computes the layout data for a single data group with flow layout. The resulting layout is saved into the input objects.
     * @param dataGroup the data group that should be laid out into a grid using the flow algorithm
     * @param subgrid the object that holds the context of the computed layouting unit
     */
    protected computeFlowLayout(dataGroup: DataGroup, subgrid: Subgrid): void;
    /**
     * Computes the layout data for a single data group with legacy layout. The resulting layout is saved into the input objects.
     *
     * The legacy layout forces the number of columns to be 4 and logs a warning if this was not the case.
     * @param dataGroup the data group that should be laid out into a grid using the legacy algorithm used in NAE versions < 4.0.0
     * @param subgrid the object that holds the context of the computed layouting unit
     */
    protected computeLegacyLayout(dataGroup: DataGroup, subgrid: Subgrid): void;
    /**
     * Lays out the fields from left to right, from top to bottom. If the width is greater than 1 and the block of flowed fields doesn't fit
     * neatly into the columns (if the width of a single field is not a divisor of column count), then the entire block of fields is
     * left aligned. The last row of fields is aligned to the left, center or right based on the data groups property.
     * If the last row cannot be aligned to the exact center it is offset one grid tile to the left.
     * @param dataGroup the data group that should be laid out into a grid
     * @param subgrid the object that holds the context of the computed layouting unit
     * @param fieldWidth the number of grid tiles, that should be occupied by each field
     */
    protected flowFields(dataGroup: DataGroup, subgrid: Subgrid, fieldWidth: number): void;
    /**
     * @param width width of the row
     * @param content the value that should be used to fill all elements in the row
     */
    protected newGridRow(width: number, content?: string): Array<string>;
    /**
     * Fills the specified rectangular area with the specified value in the provided grid.
     *
     * If the specified area contains values other than the empty string an error will be thrown.
     * @param grid the grid that should be modified
     * @param y the 0 based row index of the top-left corner of the filled area
     * @param x the 0 based column index of the top-left corner of the filled area
     * @param width the width of the filled area
     * @param value the value that is set into every cell of the grid inside the specified area
     * @param height the height of the filled area
     * @param checkOccupants whether the filled area should be checked for other elements.
     * If a check is performed an error will be thrown if elements are found.
     */
    protected occupySpace(grid: Array<Array<string>>, y: number, x: number, width: number, value: string, height?: number, checkOccupants?: boolean): void;
    /**
     * @param index the index of the checked element within its data group
     * @param dataGroup the checked data group
     * @param fieldsPerRow the number of fields that is required to fill an entire row
     * @returns whether the field at the given index is within the last row of elements of the provided data group.
     * Note that if the last row contains enough elements to be completely filled this method returns `false`.
     */
    protected isLastRow(index: number, dataGroup: DataGroup, fieldsPerRow: number): boolean;
    /**
     * Destroys the previous content and pushes the new content into the stream
     * @param content the new content
     */
    protected renderContent(content?: Array<Subgrid>): void;
    protected createSubgridId(dataGroup: DataGroup): string;
    trackBySubgridFn: (index: number, subgrid: Subgrid) => string;
    trackByFn: (index: number, element: DatafieldGridLayoutElement) => string;
    protected trackByDatafields(index: number, element: DatafieldGridLayoutElement): string;
    protected shouldResolveTaskRefData(dataRef: DataField<any>): boolean;
    protected hasRequiredComponentProperty(component: DataRefComponent, propertyName: string, propertyValue: string): boolean;
    static ɵfac: i0.ɵɵFactoryDeclaration<AbstractTaskContentComponent, [null, null, null, null, { optional: true; }, { optional: true; }]>;
    static ɵcmp: i0.ɵɵComponentDeclaration<AbstractTaskContentComponent, "ncc-abstract-task-content", never, { "noDataText": { "alias": "noDataText"; "required": false; }; "noDataIcon": { "alias": "noDataIcon"; "required": false; }; "displayNoDataIcon": { "alias": "displayNoDataIcon"; "required": false; }; }, { "taskEvent": "taskEvent"; }, never, never, false, never>;
}
